// Универсальная система подключения модулей для wpanel
// Создано для удобной работы с большими проектами
//
// ПРИМЕРЫ ИСПОЛЬЗОВАНИЯ:
//
// Основные функции:
// include("js/math_utils.js");                    // подключить файл
// include("js/helpers.js", true);                 // принудительная перезагрузка
// includeAll(["js/utils.js", "js/constants.js"]); // несколько файлов сразу
//
// Управление и отладка:
// includeDebug(true);                             // включить отладку
// includeShowCache();                             // показать загруженные файлы  
// includeClearCache();                            // очистить кеш
// includeIsLoaded("js/utils.js");                 // проверить загружен ли файл
// includeReloadAll();                             // перезагрузить все файлы
//
// Пример организации больших проектов:
// include("js/corel_constants.js");
// include("js/math_helpers.js"); 
// include("js/curve_utils.js");
// include("js/ui_helpers.js");
// include("gbx/modules/box_generator.js");
// include("gbx/modules/fefco_library.js");
//
// ФИШКИ:
// 🧠 Умное кеширование - файл загружается только один раз
// 🔧 Автонормализация путей - работает с "/" и "\"
// ⚡ Быстрая работа - повторные вызовы мгновенные
// 🐛 Отладка - логи загрузки в Output
// 📦 Совместимость с JavaScript 1.5 и VBA

// Кеш загруженных файлов для производительности
var _includedFiles = {};
var _includeDebug = false; // включить для отладки

// Главная функция подключения файлов
function include(filePath, forceReload) {
    if (forceReload === undefined) forceReload = false;
    
    // Нормализуем путь (заменяем / на \ для Windows)
    var normalizedPath = filePath.replace(/\//g, "\\");
    
    // Проверяем кеш (если не принудительная перезагрузка)
    if (!forceReload && _includedFiles[normalizedPath]) {
        if (_includeDebug) {
            dbg("Include: файл уже загружен из кеша - " + filePath);
        }
        return true;
    }
    
    try {
        if (_includeDebug) {
            dbg("Include: загружаем файл - " + filePath);
        }
        
        // Читаем файл через проверенную VBA функцию
        var fileContent = vbReadFileUTF(filePath);
        
        if (fileContent && fileContent.length > 0) {
            // Выполняем код
            eval(fileContent);
            
            // Сохраняем в кеш
            _includedFiles[normalizedPath] = true;
            
            if (_includeDebug) {
                dbg("Include: файл успешно загружен - " + filePath);
            }
            
            return true;
        } else {
            dbg("Include: ОШИБКА - файл пустой или не найден: " + filePath);
            return false;
        }
        
    } catch (e) {
        var error_string = "Include: ОШИБКА при загрузке " + filePath + " - " + e.message;
        if (e.name) { error_string += "\n  - Тип: " + e.name; }
        if (e.number) { error_string += "\n  - Код: " + e.number; } // MS JScript specific
        if (e.description) { error_string += "\n  - Описание: " + e.description; } // MS JScript specific
        if (e.lineNumber) { error_string += "\n  - Строка: " + e.lineNumber; }
        if (e.stack) { error_string += "\n  - Стек: " + e.stack; }
        dbg(error_string);
        return false;
    }
}

// Функция для подключения нескольких файлов сразу
function includeAll(fileArray, forceReload) {
    var results = [];
    var allSuccess = true;
    
    for (var i = 0; i < fileArray.length; i++) {
        var result = include(fileArray[i], forceReload);
        results.push(result);
        if (!result) allSuccess = false;
    }
    
    return {
        success: allSuccess,
        results: results
    };
}

// Функция для перезагрузки всех подключенных файлов
function includeReloadAll() {
    var paths = [];
    for (var path in _includedFiles) {
        paths.push(path);
    }
    
    // Очищаем кеш
    _includedFiles = {};
    
    // Перезагружаем все
    return includeAll(paths, true);
}

// Функция для отладки - показывает список загруженных файлов
function includeShowCache() {
    var loadedFiles = [];
    for (var path in _includedFiles) {
        loadedFiles.push(path);
    }
    
    dbg("Include: загружено файлов: " + loadedFiles.length);
    for (var i = 0; i < loadedFiles.length; i++) {
        dbg("  - " + loadedFiles[i]);
    }
    
    return loadedFiles;
}

// Включение/выключение отладки
function includeDebug(enable) {
    _includeDebug = enable;
    dbg("Include: отладка " + (enable ? "включена" : "выключена"));
}

// Очистка кеша
function includeClearCache() {
    var count = Object.keys(_includedFiles).length;
    _includedFiles = {};
    dbg("Include: кеш очищен (" + count + " файлов)");
}

// Проверка, загружен ли файл
function includeIsLoaded(filePath) {
    var normalizedPath = filePath.replace(/\//g, "\\");
    return _includedFiles[normalizedPath] === true;
}

// Функция для загрузки HTML файлов (по аналогии с include)
function includeHTML(filePath, targetElementId) {
    // Нормализуем путь
    var normalizedPath = filePath.replace(/\//g, "\\");
    
    try {
        if (_includeDebug) {
            dbg("IncludeHTML: загружаем HTML - " + filePath);
        }
        
        // Читаем файл через проверенную VBA функцию
        var fileContent = vbReadFileUTF(filePath);
        
        if (fileContent && fileContent.length > 0) {
            // Вставляем HTML в указанный элемент
            var targetElement = document.getElementById(targetElementId);
            if (targetElement) {
                targetElement.innerHTML = fileContent;
                
                if (_includeDebug) {
                    dbg("IncludeHTML: HTML успешно загружен в " + targetElementId);
                }
                
                return true;
            } else {
                dbg("IncludeHTML: ОШИБКА - элемент не найден: " + targetElementId);
                return false;
            }
        } else {
            dbg("IncludeHTML: ОШИБКА - файл пустой или не найден: " + filePath);
            return false;
        }
        
    } catch (e) {
        dbg("IncludeHTML: ОШИБКА при загрузке " + filePath + " - " + e.message);
        return false;
    }
} 